/* Mugurel Andreica
   O(N * sqrt(N)) */

#include <stdio.h>
#include <string.h>
#include <math.h>
#include <vector>

using namespace std;

#define NMAX 200100

#define max(a, b) ((a) >= (b) ? (a) : (b))
#define min(a, b) ((a) <= (b) ? (a) : (b))

vector<int> vec[NMAX], nodes[NMAX];
int color[NMAX];
int N, K;

void read_input() {
	int i, x, y;
	freopen("kdist.in", "r", stdin);
	scanf("%d %d", &N, &K);
	for (i = 1; i < N; i++) {
		scanf("%d %d", &x, &y);
		vec[x].push_back(y);
		vec[y].push_back(x);
	}
	for (i = 1; i <= N; i++) {
		scanf("%d", &color[i]);
		nodes[color[i]].push_back(i);
	}
}

char visited[NMAX];
int q[NMAX], level[NMAX];
int size_limit;

void bfs() {
	int i, j, li, ls;

	q[li = ls = 1] = 1;
	visited[1] = 1;
	level[1] = 0;

	while (li <= ls) {
		i = q[li];
		for (j = vec[i].size() - 1; j >= 0; j--)
			if (!visited[vec[i][j]]) {
				ls++;
				q[ls] = vec[i][j];
				visited[q[ls]] = 1;
				level[q[ls]] = 1 + level[i];
			}
		li++;
	}
}

int euler[2 * NMAX], pstart[NMAX], pfin[NMAX], M;

void dfs(int x) {
	int y;

	M++;
	euler[M] = level[x];
	pstart[x] = pfin[x] = M;
	visited[x] = 1;

	for (y = vec[x].size() - 1; y >= 0; y--) {
		if (level[vec[x][y]] > level[x] && !visited[vec[x][y]]) {
			dfs(vec[x][y]);
			M++;
			euler[M] = level[x];
			pfin[x] = M;
		}
	}
}

#define LOGMAX 18

int bit[LOGMAX], bitidx[2 * NMAX];
int vminleft[2 * NMAX][LOGMAX], vminright[2 * NMAX][LOGMAX];

void LCAprecompute() {
	int i, j, p;

	M = 0;
	memset(visited, 0, sizeof(visited));
	dfs(1);

	bit[0] = 1;
	for (i = 1; i < LOGMAX; i++)
		bit[i] = bit[i - 1] << 1;

	for (i = 1; i <= M; i++) {
		vminleft[i][0] = vminright[i][0] = euler[i];
		//fprintf(stderr, "euler(%d)=%d\n", i, euler[i]);
	}

	//for (i = 1; i <= N; i++)
	//	fprintf(stderr, "pstart(%d)=%d, pfin(%d)=%d\n", i, pstart[i], i, pfin[i]);
	
	for (j = 1; j < LOGMAX; j++) {
		for (i = 1; i <= M; i++) {
			vminleft[i][j] = vminleft[i][j - 1];
			p = i - bit[j - 1];
			if (p >= 1)
				vminleft[i][j] = min(vminleft[i][j], vminleft[p][j - 1]);

			vminright[i][j] = vminright[i][j - 1];
			p = i + bit[j - 1];
			if (p <= M)
				vminright[i][j] = min(vminright[i][j], vminright[p][j - 1]);
		}
	}

	bitidx[1] = j = 0;
	for (i = 2; i <= M; i++) {
		if (j < LOGMAX - 1 && i == bit[j + 1])
			j++;
		bitidx[i] = j;
	}
}

int LCAlevel(int i, int j) {
	int pmin = min(pstart[i], pstart[j]);
	int pmax = max(pfin[i], pfin[j]);
	int bidx = bitidx[pmax - pmin + 1];
	return min(vminright[pmin][bidx], vminleft[pmax][bidx]);
}

long long brute_force(int c) {
	int i, j, lcalevel;
	long long sum = 0;

	for (i = nodes[c].size() - 1; i >= 1; i--)
		for (j = i - 1; j >= 0; j--) {
			lcalevel = LCAlevel(nodes[c][i], nodes[c][j]);
			sum += (level[nodes[c][i]] + level[nodes[c][j]] - 2 * lcalevel);
		}
	
		return sum;
}

long long sumclosed[NMAX], sumopen[NMAX], cnt[NMAX];

long long dp(int c) {
	int i, j, p;
	long long x;

	for (p = N; p >= 1; p--) {
		i = q[p];
		cnt[i] = 0;
		sumclosed[i] = sumopen[i] = 0;

		for (j = vec[i].size() - 1; j >= 0; j--)
			if (level[vec[i][j]] > level[i]) {
				sumclosed[i] += sumclosed[vec[i][j]];
				sumopen[i] += (sumopen[vec[i][j]] + cnt[vec[i][j]]);
				cnt[i] += cnt[vec[i][j]];
			}

		x = 0;
		for (j = vec[i].size() - 1; j >= 0; j--)
			if (level[vec[i][j]] > level[i]) {
				x += (sumopen[vec[i][j]] + cnt[vec[i][j]]) * (cnt[i] - cnt[vec[i][j]]) +
					(sumopen[i] - sumopen[vec[i][j]] - cnt[vec[i][j]]) * cnt[vec[i][j]];
			}
		sumclosed[i] += x / 2;

		if (color[i] == c) {
			cnt[i]++;
			sumclosed[i] += sumopen[i];
		}		
	}

	return sumclosed[q[1]];
}

void compute() {
	int c;
	bfs();
	LCAprecompute();
	size_limit = max((int) sqrt(N), 1);
	//size_limit = N;
	
	freopen("kdist.out", "w", stdout);
	for (c = 1; c <= K; c++) {
		if (nodes[c].size() <= size_limit)
			printf("%lld\n", brute_force(c));
		else
			printf("%lld\n", dp(c));
	}
}

int main() {
	read_input();
	compute();
	return 0;
}
